from twisted.internet.defer import inlineCallbacks, returnValue

from juju.state.errors import NoMatchingEndpoints, AmbiguousRelation
from juju.state.relation import RelationStateManager
from juju.state.service import ServiceStateManager


@inlineCallbacks
def add_relation(context, *descriptors):

    relation_state_manager = RelationStateManager(context.client)
    service_state_manager = ServiceStateManager(context.client)
    endpoint_pairs = yield service_state_manager.join_descriptors(
            *descriptors)

    context.log.debug("Endpoint pairs: %s", endpoint_pairs)

    if len(endpoint_pairs) == 0:
        raise NoMatchingEndpoints()
    elif len(endpoint_pairs) > 1:
        for pair in endpoint_pairs[1:]:
            if not (pair[0].relation_name.startswith("juju-") or
                    pair[1].relation_name.startswith("juju-")):
                raise AmbiguousRelation(descriptors, endpoint_pairs)

    # At this point we just have one endpoint pair. We need to pick
    # just one of the endpoints if it's a peer endpoint, since that's
    # our current API - join descriptors takes two descriptors, but
    # add_relation_state takes one or two endpoints. TODO consider
    # refactoring.
    endpoints = endpoint_pairs[0]
    if endpoints[0] == endpoints[1]:
        endpoints = endpoints[0:1]
    relation, svc_rels = yield relation_state_manager.add_relation_state(
        *endpoints)
    context.log.info("Added %s relation to all service units.",
                     endpoints[0].relation_type)

    yield relation.load()
    returnValue({'id': relation.internal_id,
                 'interface': relation.interface,
                 'scope': relation.scope,
                 'endpoints': relation.endpoints})
