from twisted.internet.defer import inlineCallbacks, returnValue
from juju.errors import JujuError
from juju.state.placement import place_unit
from juju.state.service import ServiceStateManager


@inlineCallbacks
def add_unit(context, service_name, num_units=1, placement="unassigned"):
    service_manager = ServiceStateManager(context.client)
    service_state = yield service_manager.get_service_state(service_name)
    if (yield service_state.is_subordinate()):
        raise JujuError("Subordinate services acquire units from "
                        "their principal service.")

    units_ids = []
    for i in range(num_units):
        unit_state = yield service_state.add_unit_state()
        yield place_unit(context.client, placement, unit_state)
        context.log.info(
            "Unit %r added to service %r",
            unit_state.unit_name, service_state.service_name)
        units_ids.append(unit_state.unit_name)

    returnValue(units_ids)
