from twisted.internet.defer import inlineCallbacks, returnValue
from juju.state.service import ServiceStateManager


@inlineCallbacks
def config_get(context, service_name):
    # Get the service
    service_manager = ServiceStateManager(context.client)
    service = yield service_manager.get_service_state(service_name)

    # Retrieve schema
    charm = yield service.get_charm_state()
    schema = yield charm.get_config()
    schema_dict = schema.as_dict()
    display_dict = {"service": service.service_name,
                    "charm": (yield service.get_charm_id()),
                    "settings": schema_dict}

    # Get current settings
    settings = yield service.get_config()
    settings = dict(settings.items())

    # Merge current settings into schema/display dict
    for k, v in schema_dict.items():
        # Display defaults for unset values.
        if k in settings:
            v['value'] = settings[k]
        else:
            v['value'] = "-Not set-"

        if 'default' in v:
            if v['default'] == settings[k]:
                v['default'] = True
            else:
                del v['default']

    returnValue(display_dict)
