from twisted.internet.defer import inlineCallbacks, returnValue

from juju.state.service import ServiceStateManager
from juju.lib.format import get_charm_formatter
from juju.lib.serializer import yaml_load


@inlineCallbacks
def config_set(context, service_name, service_options, data=None):
    # Get the service and the charm
    service_manager = ServiceStateManager(context.client)
    service = yield service_manager.get_service_state(service_name)
    charm = yield service.get_charm_state()
    charm_format = (yield charm.get_metadata()).format
    formatter = get_charm_formatter(charm_format)

    # Use the charm's ConfigOptions instance to validate the
    # arguments to config_set. Invalid options passed to this method
    # will thrown an exception.
    if not service_options and data:
        options = yaml_load(data)
    elif isinstance(service_options, dict):
        options = service_options
    else:
        options = formatter.parse_keyvalue_pairs(service_options)

    config = yield charm.get_config()
    options = config.validate(options)

    # Apply the change
    state = yield service.get_config()
    state.update(options)
    yield state.write()

    returnValue(True)
