from twisted.internet.defer import inlineCallbacks, returnValue

from juju.state.environment import EnvironmentStateManager
from juju.state.machine import MachineStateManager
from juju.state.service import ServiceStateManager


@inlineCallbacks
def constraints_get(context, entity_names=()):
    if not entity_names:
        esm = EnvironmentStateManager(context.client)
        context.log.info("Fetching constraints for environment")
        constraints = yield esm.get_constraints()
        returnValue(dict(constraints))

    msm = MachineStateManager(context.client)
    ssm = ServiceStateManager(context.client)
    result = {}

    for name in entity_names:
        if name.isdigit():
            kind = "machine"
            entity = yield msm.get_machine_state(name)
        elif "/" in name:
            kind = "service unit"
            entity = yield ssm.get_unit_state(name)
        else:
            kind = "service"
            entity = yield ssm.get_service_state(name)
            context.log.info("Fetching constraints for %s %s", kind, name)
            constraints = yield entity.get_constraints()
            result[name] = dict(constraints)
    returnValue(result)
