from twisted.internet.defer import inlineCallbacks, returnValue

from juju.state.environment import EnvironmentStateManager
from juju.state.service import ServiceStateManager


@inlineCallbacks
def constraints_set(context, service_name, constraint_strs):
    # XXX / TODO : provider constraints require privileged access to
    # the provider, which the REST api shouldn't have ideally.

    # Perhaps we could cache serialize provider constraints with
    # periodic update. Also comes up do to frequence of constraint set
    # access for deployment.
    constraint_set = yield context.provider.get_constraint_set()
    constraints = constraint_set.parse(constraint_strs)

    if service_name is None:
        esm = EnvironmentStateManager(context.client)
        yield esm.set_constraints(constraints)
    else:
        ssm = ServiceStateManager(context.client)
        service = yield ssm.get_service_state(service_name)
        yield service.set_constraints(constraints)

    returnValue(True)
