from twisted.internet.defer import inlineCallbacks, returnValue

from juju.state.errors import UnsupportedSubordinateServiceRemoval
from juju.state.service import ServiceStateManager
from juju.state.relation import RelationStateManager


@inlineCallbacks
def destroy_service(context, service_name):
    service_manager = ServiceStateManager(context.client)
    service_state = yield service_manager.get_service_state(service_name)

    if (yield service_state.is_subordinate()):
        # We can destroy the service if does not have relations.
        # That implies that principals have already been torn
        # down (or were never added).
        relation_manager = RelationStateManager(context.client)
        relations = yield relation_manager.get_relations_for_service(
            service_state)

        if relations:
            principal_service = None
            # if we don't have a container we can destroy the subordinate
            # (revisit in the future)
            for relation in relations:
                if relation.relation_scope != "container":
                    continue
                services = yield relation.get_service_states()
                remote_service = [s for s in services if s.service_name !=
                                     service_state.service_name][0]
                if not (yield remote_service.is_subordinate()):
                    principal_service = remote_service
                    break

            if principal_service:
                raise UnsupportedSubordinateServiceRemoval(
                    service_state.service_name,
                    principal_service.service_name)

    yield service_manager.remove_service_state(service_state)
    context.log.info("Service %r destroyed.", service_state.service_name)
    returnValue(True)
