from twisted.internet.defer import inlineCallbacks, returnValue

from juju.state.errors import (
    UnsupportedSubordinateServiceRemoval, ServiceUnitStateNotFound)
from juju.state.service import ServiceStateManager, parse_service_name


@inlineCallbacks
def remove_unit(context, unit_names):
    service_manager = ServiceStateManager(context.client)
    for unit_name in unit_names:
        service_name = parse_service_name(unit_name)
        service_state = yield service_manager.get_service_state(
            service_name)
        try:
            unit_state = yield service_state.get_unit_state(unit_name)
        except ServiceUnitStateNotFound:
            context.log.warning("Unit %r does not exist", unit_name)
            continue
        if (yield service_state.is_subordinate()):
            container = yield unit_state.get_container()
            raise UnsupportedSubordinateServiceRemoval(
                unit_state.unit_name,
                container.unit_name)

        yield service_state.remove_unit_state(unit_state)
        context.log.info("Unit %r removed from service %r",
                 unit_state.unit_name, service_state.service_name)

    returnValue(True)
