from twisted.internet.defer import inlineCallbacks, returnValue

from juju.errors import CannotTerminateMachine
from juju.state.errors import MachineStateNotFound
from juju.state.machine import MachineStateManager


@inlineCallbacks
def terminate_machine(context, machine_ids):
    """Terminates the machines in `machine_ids`.

    Like the underlying code in MachineStateManager, it's permissible
    if the machine ID is already terminated or even never running. If
    we determine this is not desired behavior, presumably propagate
    that back to the state manager.

    XXX However, we currently special case support of not terminating
    the "root" machine, that is the one running the provisioning
    agent. At some point, this will be managed like any other service,
    but until then it seems best to ensure it's not terminated at this
    level.
    """
    terminated_machine_ids = []
    machine_state_manager = MachineStateManager(context.client)

    for machine_id in machine_ids:
        if machine_id == 0:
            raise CannotTerminateMachine(
                0, "environment would be destroyed")
        removed = yield machine_state_manager.remove_machine_state(
            machine_id)
        if not removed:
            raise MachineStateNotFound(machine_id)
        terminated_machine_ids.append(machine_id)

    if terminated_machine_ids:
        context.log.info(
            "Machines terminated: %s",
                ", ".join(str(id) for id in terminated_machine_ids))

    returnValue(True)
