import json
import urllib

from twisted.internet.defer import inlineCallbacks, returnValue

from juju.rapi.context import APIContext

from juju.state.tests.common import StateTestBase
from juju.providers.dummy import MachineProvider
from juju.charm.repository import CS_STORE_URL, resolve

from juju.charm.tests.test_repository import RepositoryTestBase
from juju.lib.mocker import ANY, MATCH

from txaws.client.ssl import VerifyingContextFactory


TEST_ENV = """
environments:
  cloudy:
    type: dummy
    default-series: precise
"""

MATCH_TLS_CTX = MATCH(lambda x: isinstance(x, VerifyingContextFactory))


class ContextTestBase(StateTestBase, RepositoryTestBase):

    @inlineCallbacks
    def setUp(self):
        yield super(ContextTestBase, self).setUp()
        self.provider = MachineProvider(
            "cloudy", {"default-series": "precise",
                       "storage-directory": self.makeDir()})
        self.context = APIContext(self.client, self.provider)
        self.context.charm_cache_dir = self.makeDir()
        yield self.client.create("/environment", TEST_ENV)

        # Mocked out on usage, wait till nesc. so non mock tests cases
        # are fine.
        self._get = None
        self._download = None

    @inlineCallbacks
    def get_charm(self, charm_spec):
        # Mock out store accessors.
        if self._get is None:
            self._get = self.mocker.replace("twisted.web.client.getPage")
        if self._download is None:
            self._download = self.mocker.replace(
                "twisted.web.client.downloadPage")
        # Find the local charm to return
        charm_name = charm_spec.split("/")[-1]
        disk_repo, disk_repo_url = resolve(
            "local:series/%s" % charm_name,
            self.unbundled_repo_path, "series")
        charm_dir = yield disk_repo.find(disk_repo_url)
        returnValue(charm_dir)

    @inlineCallbacks
    def mock_store_charms(self, charm_specs):
        for cs in charm_specs:
            charm_dir = yield self.get_charm(cs)
            self._mock_store_charm(cs, charm_dir)

    def _mock_store_charm(self, charm_spec, charm_dir):
        # Mock proper api results
        if not ":" in charm_spec:
            charm_spec = "cs:%s" % charm_spec
        url_frag = urllib.quote(charm_spec)
        info_url = CS_STORE_URL + "/charm-info?charms=%s" % url_frag
        self._get(info_url, contextFactory=MATCH_TLS_CTX)

        revision = charm_dir.get_revision()
        charm_bundle = charm_dir.as_bundle()
        self.mocker.result(
            json.dumps(
                {charm_spec:
                 {"revision": revision,
                  "sha256": charm_bundle.compute_sha256(),
                  "errors": [],
                  "warnings": []}}))

        # Sigh, charm dir->bundle tempfile management is broken imo.
        # we can't just read this in the closure, since the charm dir
        # which owns the bundle's tempfile is gone.
        charm_contents = open(charm_bundle.path).read()

        def dl(charm_url, download_path, contextFactory):
            with open(download_path, 'wb') as fh:
                fh.write(charm_contents)

        _, fragment = charm_spec.split(":")
        download_url = "".join((
            CS_STORE_URL, "/charm/", fragment, "-", str(revision)),)
        self._download(download_url, ANY, contextFactory=MATCH_TLS_CTX)
        self.mocker.call(dl)
