from twisted.internet.defer import inlineCallbacks
from juju.rapi.tests.common import ContextTestBase
from juju.rapi.rest import get_service, get_charm, get_endpoints


class RestAPITest(ContextTestBase):

    @inlineCallbacks
    def test_get_endpoints(self):
        self.mock_store_charms(["precise/wordpress", "precise/mysql"])
        self.mocker.replay()
        yield self.context.deploy("precise/mysql")
        yield self.context.deploy("wordpress")

        self.assertEqual(
            (yield get_endpoints(self.context)),
            {'mysql': {'provides': [{'interface': u'mysql',
                                     'limit': None,
                                     'name': u'server',
                                     'optional': False,
                                     'scope': 'global'}],
                       'requires': []},
             'wordpress': {'provides': [{'interface': u'http',
                                         'limit': None,
                                         'name': u'url',
                                         'optional': False,
                                         'scope': 'global'}],
                           'requires': [{'interface': u'varnish',
                                         'limit': 2,
                                         'name': u'cache',
                                         'optional': True,
                                         'scope': 'global'},
                                        {'interface': u'mysql',
                                         'limit': 1,
                                         'name': u'db',
                                         'optional': False,
                                         'scope': 'global'}]}}
        )

        self.assertEqual(
            (yield get_endpoints(self.context, ['mysql'])),
            {'mysql': {'provides': [{'interface': u'mysql',
                                     'limit': None,
                                     'name': u'server',
                                     'optional': False,
                                     'scope': 'global'}],
                       'requires': []}})

    @inlineCallbacks
    def test_get_service(self):
        self.mock_store_charms(["precise/wordpress", "precise/mysql"])
        self.mocker.replay()
        yield self.context.deploy("precise/mysql")
        yield self.context.deploy("wordpress")
        yield self.context.add_relation("wordpress", "mysql")

        self.assertEqual(
            (yield get_service(self.context, "wordpress")),
            {'name': 'wordpress',
             'charm': 'cs:precise/wordpress-3',
             'config': {'blog-title': 'My Title'},
             'exposed': False,
             'constraints': {
                 'ubuntu-series': 'precise',
                 'provider-type': 'dummy'},
             'rels': [
                 {'ident': 'db:0',
                  'name': 'db',
                  'role': 'client',
                  'endpoint': 'mysql',
                  'scope': 'global'}]
             })

    @inlineCallbacks
    def test_get_charm(self):
        self.mock_store_charms(["precise/wordpress"])
        self.mocker.replay()
        yield self.context.deploy("precise/wordpress")
        self.assertEqual(
            (yield get_charm(self.context, "cs:precise/wordpress-3")),
            {'config': {'options': {
                'blog-title': {
                    'default': 'My Title',
                    'description': 'A descriptive title used for the blog.',
                    'type': 'string'}}},
             'name': 'wordpress',
             'peers': None,
             'provides': {u'url': {'interface': u'http',
                                   'limit': None,
                                   'optional': False,
                                   'scope': 'global'}},
             'requires': {u'cache': {'interface': u'varnish',
                                     'limit': 2,
                                     'optional': True,
                                     'scope': 'global'},
                          u'db': {'interface': u'mysql',
                                  'limit': 1,
                                  'optional': False,
                                  'scope': 'global'}},
             'revision': 3,
             'subordinate': False,
             'url': 'cs:precise/wordpress-3'}
            )
